import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 * Clase que crea una ventana en la que se abrirán un mensaje original
 * y su mensaje falso con mismo hash.
 * @author Jesús Vidal Panalés
 *
 */
public class AbrirMensajes extends JFrame
{
	private static final long serialVersionUID = 1;
	private JPanel panel;
	private JLabel etiquetaOriginal;
	private JLabel etiquetaFalso;
	private JTextPane mensajeOriginal;
	private JTextPane mensajeFalso;
	private JScrollPane panelOriginal;
	private JScrollPane panelFalso;
	private JButton botonCerrar;
	private File original;
	private File falso;
	
	/**
	 * Clase que implementa el ActionListener de Cerrar la ventana
	 * y liberarla de la memoria.
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class Cerrar implements ActionListener
	{
		/**
		 * Método necesario para procesar el Evento, que en este caso cerrará la ventana
		 */
		public void actionPerformed(ActionEvent e)
		{
			JButton cerrar = (JButton) e.getSource();
			JFrame jc = (JFrame)cerrar.getParent().getParent().getParent().getParent().getParent();
			jc.dispose();
		}
	}

	/**
	 * Método que lee el contenido de un archivo y lo devuelve en un String
	 * @param file El archivo que se quiere abrir para leer
	 * @return La cadena resultante del texto del mensaje
	 */
	private String contenido (File file)
	{
		String contenido = "";
		String texto = "";
		try
		{
			BufferedReader mensaje = new BufferedReader(new FileReader(file));
			while ((texto = mensaje.readLine()) != null)
	    	{
	    		contenido += texto;	
	    	}
		}
		catch (FileNotFoundException fnfe)
		{
			fnfe.printStackTrace();
		}
		catch (IOException ioe)
		{
			ioe.printStackTrace();
		}
		return contenido;
	}
	
	/**
	 * Constructor de la clase que abre una ventana con el mensaje original
	 * y su mensaje falso con el mismo hash
	 * @param original El archivo original
	 * @param falso El archivo falso
	 */
	public AbrirMensajes (File original, File falso)
	{
		this.original = original;
		this.falso = falso;
		
		this.etiquetaOriginal = new JLabel ("Mensaje original: "+original.getName());
		this.etiquetaOriginal.setPreferredSize(new Dimension (330,25));
		this.etiquetaOriginal.setHorizontalTextPosition(SwingConstants.CENTER);
		this.etiquetaOriginal.setHorizontalAlignment(SwingConstants.CENTER);
		this.etiquetaOriginal.setBorder(BorderFactory.createLineBorder(Color.white));
		this.etiquetaOriginal.setForeground(Color.white);

		this.etiquetaFalso = new JLabel ("Mensaje falso: "+falso.getName());
		this.etiquetaFalso.setPreferredSize(new Dimension (330,25));
		this.etiquetaFalso.setHorizontalTextPosition(SwingConstants.CENTER);
		this.etiquetaFalso.setHorizontalAlignment(SwingConstants.CENTER);
		this.etiquetaFalso.setBorder(BorderFactory.createLineBorder(Color.white));
		this.etiquetaFalso.setForeground(Color.white);

		this.mensajeOriginal = new JTextPane();
		this.mensajeOriginal.setPreferredSize(new Dimension (330,250));
		
		this.mensajeFalso = new JTextPane();
		this.mensajeFalso.setPreferredSize(new Dimension (330,250));
		
		this.panelOriginal = new JScrollPane(this.mensajeOriginal);
		this.panelOriginal.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		this.panelOriginal.setPreferredSize(new Dimension(330, 250));
		this.panelOriginal.setBorder
        (
        		BorderFactory.createCompoundBorder
        		(
        				BorderFactory.createEmptyBorder(5,5,5,5),
        				this.panelOriginal.getBorder()
                )
        );
		this.panelOriginal.setBackground(Color.black);
	
		this.panelFalso = new JScrollPane(mensajeFalso);
		this.panelFalso.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		this.panelFalso.setPreferredSize(new Dimension(330, 250));
		this.panelFalso.setBorder
        (
        		BorderFactory.createCompoundBorder
        		(
        				BorderFactory.createEmptyBorder(5,5,5,5),
        				this.panelFalso.getBorder()
                )
        );
		this.panelFalso.setBackground(Color.black);
		
		ImageIcon botonCerrarNormal = new ImageIcon(getClass().getClassLoader().getResource("images/botonCerrarNormal.png"));
		ImageIcon botonCerrarPulsado = new ImageIcon(getClass().getClassLoader().getResource("images/botonCerrarPulsado.png"));
		Dimension boton = new Dimension (botonCerrarNormal.getIconWidth(), botonCerrarNormal.getIconHeight());

		this.botonCerrar = new JButton();
		this.botonCerrar.setIcon(botonCerrarNormal);
		this.botonCerrar.addActionListener(new Cerrar());
		this.botonCerrar.setToolTipText("Cierra la ventana");
		this.botonCerrar.setPreferredSize(boton);
		this.botonCerrar.setPressedIcon(botonCerrarPulsado);
		this.botonCerrar.setBorderPainted(false);
		
		this.mensajeOriginal.setText(contenido(this.original));
		this.mensajeFalso.setText(contenido(this.falso));
				
		this.panel = new JPanel();
		this.panel.setPreferredSize(new Dimension (700,350));
		this.panel.add(this.etiquetaOriginal);
		this.panel.add(this.etiquetaFalso);
		this.panel.add(this.panelOriginal);
		this.panel.add(this.panelFalso);
		this.panel.add(this.botonCerrar);
		this.panel.setBackground(Color.black);
		
	    this.getContentPane().add(this.panel);

	    setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		try
		{
			UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		
		this.setIconImage(new ImageIcon(getClass().getClassLoader().getResource("images/iconoHash.png")).getImage());
		this.getContentPane().setBackground(Color.BLACK);
		this.setTitle("Mensaje original y falso con mismo hash");
		this.setMinimumSize(new Dimension(700,350));
		this.setLocation((int)Toolkit.getDefaultToolkit().getScreenSize().getWidth()/2-this.getSize().width/2,(int)Toolkit.getDefaultToolkit().getScreenSize().getHeight()/2-this.getSize().height/2);
		this.setResizable(false);
		this.pack();
		this.setVisible(true);
	}
}
