
import java.util.*;

/**
 * Clase de implementación del algoritmo SDES
 * @author Jesús Vidal Panalés
 *
 */
public class SDES {
	private int rondas; //numero de rondas
	private String mensaje; //mensaje de entrada en texto de caracteres (convertir a bits)
	private String clave_cifrado; // clave de 9 bits para cifrar
	private int bits_clave; //cantidad de bits para la clave (por defecto: 9)
	private int[] expandir = {1, 2, 4, 3, 4, 3, 5, 6}; //para la funcion de expansion
	private String[][] sbox1 = { //para funcion sbox1
			{"101", "010", "001", "110", "011", "100", "111", "000"},
			{"001", "100", "110", "010", "000", "111", "101", "011"}
	};
	private String [][]sbox2 = { //para funcion sbox2
			{"100", "000", "110", "101", "111", "001", "011", "010"},
			{"101", "011", "000", "111", "110", "010", "001", "100"}
	};

	/**
	 * Constructor SDES
	 * @param m Mensaje de texto para cifrar
	 * @param r Cantidad de rondas
	 * @param clave Clave del cifrado de 9 bits
	 */
	public SDES (String m, int r, String clave)
	{
		this.mensaje = m;
		this.rondas = r;
		this.clave_cifrado = clave;
		this.bits_clave = 9;
	}

	/**
	 * Convierte una cadena de bits a BitSet
	 * @param cadena Cadena de bits
	 * @return BitSet correspondiente a la cadena de bits
	 */
	private BitSet cadenaToBitSet (String cadena)
	{
		BitSet bits = new BitSet (cadena.length());
		for (int i=0; i<cadena.length(); i++)
		{
			if (cadena.charAt(i)=='1') bits.set(i);
			else bits.clear(i);
		}
		
		return bits;
	}

	/**
	 * Convierte un BitSet a una cadena en binario
	 * @param b El mensaje en BitSet
	 * @param n El tamaño que tendrá la cadena
	 * @return La cadena de bits de tamaño n
	 */
	private String BitSetToCadena (BitSet b, int n)
	{
		String tmp = "";
		for (int i = 0; i < n; i++)
		{
			if (b.get(i)) tmp = tmp + "1";
			else tmp = tmp + "0";
		}

		return tmp;
	}
	
	/**
	 * Función que expande una entrada de 6 bits a 8 bits
	 * @param entrada Cadena de 6 bits
	 * @return Cadena de 8 bits tras la expansión de la entrada
	 */
	private String expansion (String entrada)
	{
		String salida = "";
		char temp[] = entrada.toCharArray();
		
		for (int i=0; i < this.expandir.length; i++)
		{
			salida = salida + temp [this.expandir[i]-1];
		}
		return salida;
	}
	
	/**
	 * Función S-Box1 que recibe una cadena de 4 bits y devuelve el valor
	 * correspondiente a la fila sacado del primer bit y la columna del
	 * resto de bits
	 * @param m Cadena de 4 bits
	 * @return La cadena correspondiente a la fila y columna de S-Box1
	 */
	private String sBox1 (String m)
	{
		String f = "";
		String c = "";
		int fila, columna;
		
		f = f + m.charAt(0); //elige la fila
		c = c + m.substring(1); //elige la columna
		fila = Integer.parseInt(f); //fila de binario a entero
		columna = Integer.parseInt(c, 2); //columna de binario a entero
		
		return this.sbox1[fila][columna];
	}

	/**
	 * Función S-Box2 que recibe una cadena de 4 bits y devuelve el valor
	 * correspondiente a la fila sacado del primer bit y la columna del
	 * resto de bits
	 * @param m Cadena de 4 bits
	 * @return La cadena correspondiente a la fila y columna de S-Box2
	 */
	private String sBox2 (String m)
	{
		String f = "";
		String c = "";
		int fila, columna;
		
		f = f + m.charAt(0); //elige le fila
		c = c + m.substring(1); //elige la columna
		fila = Integer.parseInt(f); //fila de binario a entero
		columna = Integer.parseInt(c, 2); //columna de binario a entero
		
		return sbox2[fila][columna];
	}
	
	/**
	 * Generador de claves Ki
	 * Dado el valor de la ronda, genera y devuelve la clave Ki correspondiente
	 * @param n La ronda para la que se genera la clave
	 * @return La clave de 8 bits
	 */
	private String claveKi (int n)
	{
		String Ki = "";

		int rondaKi = n % this.bits_clave; //Por si se introducen rondas mayores de la
										   //clave de 9 bits, se hace el módulo y así, 
										   //es más fácil conocer qué clave se genera
										   //para la ronda. Se reinicializa, se empieza
										   //otra vez por la clave inicial, cuando se
										   //sobrepasa:
										   // K0 = K9, K1 = K10, ....
		
		if (rondaKi == 0)
		{
			for (int i = rondaKi; i < this.bits_clave-1; i++)
			{
				Ki = Ki + this.clave_cifrado.charAt(i);
			}
		}
		else
		{
			for (int i = rondaKi; i < this.bits_clave; i++)
			{
				Ki = Ki + this.clave_cifrado.charAt(i);
			}
			if (Ki.length() < this.bits_clave-1)
			{
				int tmp = (this.bits_clave-1) - Ki.length();
				for (int i = 0; i < tmp; i++)
				{
					Ki = Ki + this.clave_cifrado.charAt(i);
				}
			}
		}

		return Ki;
	}

	/**
	 * Dada una cadena de bits, divide en dos el mensaje de entrada de n bits.
	 * Se utiliza para dividir un mensaje de 12 bits de entrada en dos mensajes
	 * de 6 bits L y R. También se utiliza para dividir un mensaje de 8 bits en
	 * la función f, en dos mensajes de 4 bits.
	 * @param m Cadena que se quiere dividir en dos
	 * @param n Tamaño para cada subcadena
	 * @return Devuelve el array con dos cadenas de bits
	 */
	private String[] mensajes_nbits (String m, int n)
	{
		String []mensajes_nbits = new String[2];
		mensajes_nbits[0] = m.substring(0,n);
		mensajes_nbits[1] = m.substring(n);

		return mensajes_nbits;
	}

	/**
	 * Función que concatena dos mensajes
	 * @param m1 Mensaje1 para concatenar
	 * @param m2 Mensaje2 para concatenar
	 * @return La cadena resultante de concatenar Mensaje1 con Mensaje2
	 */
	private String unir_mensajes (String m1, String m2)
	{
		return m1.concat(m2);
	}
	
	/**
	 * Algoritmo de la funnción f.
	 * Entra un mensaje R de 6 bits, se expande a un mensaje de 8 bits.
	 * Se divide en dos mensajes de 4 bits, y cada mensaje se usa para
	 * extraer el valor correspondiente de su S-Box correspondiente.
	 * Finalmente se une la salida de cada S-Box en un único mensaje.
	 * @param m Mensaje R de entrada a la función
	 * @param n La ronda del cifrado
	 * @return El mensaje resultante tras el proceso de la función
	 */
	private String funcion (String m, int n)
	{
		String tmp = "";
		String []partes = new String[2];
		String clave = "";
		BitSet mensajeE;
		BitSet Ki;
		tmp = this.expansion (m); // de 6 a 8 bits

		mensajeE = this.cadenaToBitSet(tmp); //convierto a BitSet
		clave = this.claveKi(n);
		Ki = this.cadenaToBitSet(clave);
		
		mensajeE.xor(Ki); //xor de mensajeE con clave Ki
		
		tmp = this.BitSetToCadena(mensajeE, tmp.length()); //paso a texto binario
		partes = this.mensajes_nbits(tmp, tmp.length()/2); // divido el mensaje en 2

		partes[0] = this.sBox1(partes[0]); //valor que devuelve sbox1
		partes[1] = this.sBox2(partes[1]); //valor que devuelve sbox2

		tmp = unir_mensajes(partes[0], partes[1]);
		
		return tmp;
	}
	
	/**
	 * Algoritmo de SDES. Realiza la llamada al cifrado para el mensaje.
	 * @return La cadena de bits resultantes del cifrado.
	 */
	public String algoritmoSDES ()
	{
		String tmp = "";
		tmp = tmp + this.cifradoSDES(this.mensaje);
		return tmp;
	}

	/**
	 * Algoritmo del cifrado SDES
	 * @param m Cadena de 12 bits a cifrar
	 * @return Devuelve una cadena de 12 bits cifrada
	 */
	private String cifradoSDES (String m)
	{
		String tmp = "";
		String []LR;
		String L = "";
		String R = "";
		String Ri = "";
		BitSet bitsL;
		BitSet bitsR;
		LR = this.mensajes_nbits(m, m.length()/2);
		
		L = LR[0];
		R = LR[1];
		
		for (int i=0; i<this.rondas; i++)
		{
			Ri = R;
			R = this.funcion(R, i);
			bitsL = new BitSet (L.length());
			bitsR = new BitSet (R.length());
			bitsL = this.cadenaToBitSet(L);
			bitsR = this.cadenaToBitSet(R);

			bitsL.xor(bitsR);

			L = this.BitSetToCadena(bitsL, L.length());
			
			tmp = L;
			L = Ri;
			R = tmp;
		}
		return this.unir_mensajes(L, R);
	}
}